//  Created by dasdom on 19.01.20.
//  Copyright © 2020 dasdom. All rights reserved.
//

import UIKit
import Combine
import LogStore
import CoreLocation

class DistanceViewController: UIViewController {
  
  @IBOutlet var distanceLabel: UILabel!
  private let locationProvider = LocationProvider()
  private var subscription: AnyCancellable?
  /*
  private var storedLocation: CLLocation?
  */
  private var storedLocation: CLLocation? {
    didSet {
      if let location = storedLocation {
        write(location)
      }
    }
  }
  private var lastDistance: CLLocationDistance = 0
  private var stillInRadius = false

  override func viewDidLoad() {
    super.viewDidLoad()
    
    locationProvider.start()
    
    subscription = locationProvider.$lastLocation
      .sink(receiveValue: updateUI)
  }
  
  override func viewWillAppear(_ animated: Bool) {
    super.viewWillAppear(animated)
    
    storedLocation = loadLocation()
  }
  
  func updateUI(for location: CLLocation?) {
    
    guard let location = location else {
      return
    }
    
    if let storedLocation = storedLocation {
      
      let distance = location.distance(from: storedLocation)
      distanceLabel.text = String(format: "%.2lf m", distance)

      if lastDistance < distance {
        view.backgroundColor = .red
      } else if lastDistance > distance {
        view.backgroundColor = .green
      }
      
      lastDistance = distance
      
      /*
      if distance < 10 {
        showAR(shouldRestore: true)
      }
      */
      if distance < 10 {
        if stillInRadius == false {
          showAR(shouldRestore: true)
        }
      } else {
        stillInRadius = false
      }
    } else {
      let coordinate = location.coordinate
      distanceLabel.text = String(format: "%.6lf, %.6lf",
                                  coordinate.latitude,
                                  coordinate.longitude)
    }
    /*
    let coordinate = location.coordinate
    distanceLabel.text = String(format: "%.6lf, %.6lf",
                                coordinate.latitude,
                                coordinate.longitude)
    */
  }

  @IBAction func setAnchor(_ sender: UIButton) {
    storedLocation = locationProvider.lastLocation
    
    showAR()
  }
  
  private func write(_ location: CLLocation?) {
    
    guard let location = location else {
      return
    }
    
    do {
      let data = try NSKeyedArchiver.archivedData(withRootObject: location,
                                                  requiringSecureCoding: true)
      try data.write(to: FileManager.locationURL())
    } catch {
      printLog("error: \(error)")
    }
  }
  
  private func loadLocation() -> CLLocation? {
    do {
      let data = try Data(contentsOf: FileManager.locationURL())
      return try NSKeyedUnarchiver.unarchivedObject(
        ofClass: CLLocation.self, from: data)
    } catch {
      printLog("error: \(error)")
    }
    return nil
  }
  
  /*
  func showAR() {
  */
  func showAR(shouldRestore: Bool = false) {
    
    stillInRadius = true
    
    if let next = storyboard?.instantiateViewController(
      withIdentifier: "ARViewController") as? ARViewController {
      
      next.shouldRestore = shouldRestore
      next.modalPresentationStyle = .fullScreen
      present(next, animated: true, completion: nil)
    }
  }
}
