//  Created by dasdom on 28.12.19.
//  Copyright © 2019 dasdom. All rights reserved.
//

import UIKit
import CoreLocation
import LogStore

class LocationProvider: NSObject,
                        CLLocationManagerDelegate,
                        ObservableObject {
/*
class LocationProvider: NSObject,
                        CLLocationManagerDelegate {
*/
  let locationManager: CLLocationManager
  /*
  var regionUpdates: [RegionUpdate] = []
  */
  var regionUpdates: [RegionUpdate] = [] {
    didSet {
      dayEntries = DayEntriesCalculator.dayEntries(from: regionUpdates)
    }
  }
  /*
  var dayEntries: [DayEntry] = []
  @Published var dayEntries: [DayEntry] = []
  */
  var max: TimeInterval = 1
  @Published var dayEntries: [DayEntry] = []  {
    didSet {
      max = dayEntries.reduce(1.0, { result, nextDayEntry in
        Swift.max(result, nextDayEntry.duration)
      })
    }
  }
  @Published var wrongAuthorization = false

  override init() {
    
    locationManager = CLLocationManager()
    
    super.init()
    
    loadRegionUpdates()

    locationManager.delegate = self
    locationManager.requestAlwaysAuthorization()
  }

  // MARK: - CLLocationManagerDelegate
  func locationManager(_ manager: CLLocationManager,
                       didChangeAuthorization status:
                       CLAuthorizationStatus) {
    
    switch status {
    case .authorizedAlways:
      printLog("success")
    case .notDetermined:
      printLog("notDetermined")
    default:
      // FIXME: add code
      wrongAuthorization = true
      break
    }
  }
  
  func locationManager(_ manager: CLLocationManager,
                       didUpdateLocations locations: [CLLocation]) {

    guard let location = locations.last else { return }
    printLog("location: \(location)")
    
    let region = CLCircularRegion(center: location.coordinate,
                                  radius: 10,
                                  identifier: "home")
    manager.startMonitoring(for: region)
  }

  func locationManager(_ manager: CLLocationManager,
                       didFailWithError error: Error) {
    
    printLog("locationManager didFailWithError: \(error)")
  }

  func locationManager(_ manager: CLLocationManager,
                       didEnterRegion region: CLRegion) {
    
    printLog("didEnterRegion: \(String(describing: region))")
    
    addRegionUpdate(type: .enter)
  }

  func locationManager(_ manager: CLLocationManager,
                       didExitRegion region: CLRegion) {
    
    printLog("didExitRegion: \(String(describing: region))")
    
    addRegionUpdate(type: .exit)
  }

  func addRegionUpdate(type: UpdateType) {
    
    let lastUpdateType = regionUpdates.last?.updateType
    if type != lastUpdateType {
      let regionUpdate = RegionUpdate(date: Date(), updateType: type)
      regionUpdates.append(regionUpdate)
      
      writeRegionUpdates()
    }
  }

  func setHome() {
    locationManager.requestLocation()
  }
  
  func writeRegionUpdates() {
    do {
      let data = try JSONEncoder().encode(regionUpdates)
      try data.write(to: FileManager.regionUpdatesDataPath(),
                     options: .atomic)
    } catch {
      printLog("error: \(error)")
    }
  }
  
  func loadRegionUpdates() {
    do {
      let data = try Data(contentsOf: FileManager.regionUpdatesDataPath())
      regionUpdates = try JSONDecoder().decode([RegionUpdate].self,
                                               from: data)
    } catch {
      printLog("error: \(error)")
    }
  }
}

