//  Created by dasdom on 01.09.19.
//  Copyright © 2019 dasdom. All rights reserved.
//

import UIKit
import MapKit

class DrawingView: UIView {

  let mapView: MKMapView
  let clearButton: UIButton
  let startStopButton: UIButton
  let shareButton: UIButton

  override init(frame: CGRect) {
    
    mapView = MKMapView()
    mapView.showsUserLocation = true
    
    clearButton = UIButton(type: .system)
    clearButton.setTitle("Clear", for: .normal)
    
    startStopButton = UIButton(type: .system)
    startStopButton.setTitle("Start", for: .normal)
    
    shareButton = UIButton(type: .system)
    shareButton.setTitle("Share", for: .normal)
    
    super.init(frame: frame)
    
    backgroundColor = .white
    
    mapView.delegate = self
    
    // Pro tip: stack views in stack views
    let buttonStackView = UIStackView(
      arrangedSubviews: [clearButton, startStopButton, shareButton])
    buttonStackView.distribution = .fillEqually
    
    let stackView = UIStackView(
      arrangedSubviews: [mapView, buttonStackView])
    stackView.axis = .vertical

    addSubview(stackView)
    
    stackView.translatesAutoresizingMaskIntoConstraints = false

    NSLayoutConstraint.activate([
      stackView.topAnchor.constraint(equalTo: topAnchor),
      stackView.leadingAnchor.constraint(equalTo: leadingAnchor),
      stackView.bottomAnchor.constraint(equalTo: bottomAnchor),
      stackView.trailingAnchor.constraint(equalTo: trailingAnchor)
      ])
  }
  
  required init?(coder aDecoder: NSCoder) {
    fatalError()
  }
  
  func addOverlay(with coordinates: [CLLocationCoordinate2D]) {
    
    mapView.removeOverlays(mapView.overlays)
    
    let overlay = MKPolyline(coordinates: coordinates,
                             count: coordinates.count)
    mapView.addOverlay(overlay)
   
    if let lastCoordinate = coordinates.last {
      
      let lastLocation = CLLocation(
        latitude: lastCoordinate.latitude,
        longitude: lastCoordinate.longitude)
      
      // Pro tip: zoom map to show the whole path
      let maxDistance = coordinates.reduce(
        100, { result, next -> Double in
          
        let nextLocation = CLLocation(latitude: next.latitude,
                                      longitude: next.longitude)
        let distance = nextLocation.distance(from: lastLocation)
        return max(result, distance)
      })
      
      let region = MKCoordinateRegion(
        center: lastCoordinate,
        latitudinalMeters: maxDistance,
        longitudinalMeters: maxDistance)
      mapView.setRegion(region, animated: true)
    }
  }
}

// MARK: MKMapViewDelegate
extension DrawingView : MKMapViewDelegate {
  func mapView(_ mapView: MKMapView,
               rendererFor overlay: MKOverlay)
    -> MKOverlayRenderer {
      
      if overlay is MKPolyline {
        let renderer = MKPolylineRenderer(overlay: overlay)
        renderer.strokeColor = .red
        renderer.lineWidth = 3
        return renderer
      }
      return MKOverlayRenderer(overlay: overlay)
  }
}
