//  Created by dasdom on 01.09.19.
//  Copyright © 2019 dasdom. All rights reserved.
//

import UIKit
import CoreLocation
import MapKit

class DrawingsViewController: UIViewController {
  
  private var locationProvider: LocationProvider? = nil
  private var coordinates: [CLLocationCoordinate2D] = []
  
  // Pro tip: use a calculated property to get the right type of the view
  private var contentView: DrawingView {
    return view as! DrawingView
  }
  
  // Pro tip: use loadView to create a view wihout interface builder
  override func loadView() {
    let contentView = DrawingView(
      frame: UIScreen.main.bounds)
    
    contentView.startStopButton.addTarget(
      self,
      action: #selector(startStop(sender:)),
      for: .touchUpInside)
    
    contentView.clearButton.addTarget(
      self,
      action: #selector(clear(sender:)),
      for: .touchUpInside)
    
    contentView.shareButton.addTarget(
      self,
      action: #selector(share(sender:)),
      for: .touchUpInside)
    
    view = contentView
  }
  
  override func viewDidLoad() {
    super.viewDidLoad()
    
    locationProvider = LocationProvider(
      updateHandler: { [weak self] location, error in
        
        print("location: \(String(describing: location))")
        
        guard let self = self else { return }
        
        guard let location = location else { return }
        self.coordinates.append(location.coordinate)
        
        self.contentView.addOverlay(with: self.coordinates)
    })
  }
  
  @objc func startStop(sender: UIButton) {
    if locationProvider?.updating ?? false {
      locationProvider?.stop()
      sender.setTitle("Start", for: .normal)
    } else {
      locationProvider?.start()
      sender.setTitle("Stop", for: .normal)
    }
  }
  
  @objc func clear(sender: UIButton) {
    coordinates.removeAll()
    contentView.addOverlay(with: coordinates)
  }
  
  @objc func share(sender: UIButton) {
    let options = MKMapSnapshotter.Options()
    options.region = contentView.mapView.region
    
    // Pro tip: get a snapshot from a map view
    let snapshotter = MKMapSnapshotter(options: options)
    snapshotter.start { (snapshot, error) in
      
      guard let snapshot = snapshot else {
          return
      }
      let image = self.imageByAddingPath(
        with: self.coordinates,
        to: snapshot)
      
      let activity = UIActivityViewController(
        activityItems: [image],
        applicationActivities: nil)
      self.present(activity, animated: true, completion: nil)
    }
  }
  
  func imageByAddingPath(
    with coordinates: [CLLocationCoordinate2D],
    to snapshot: MKMapSnapshotter.Snapshot)
    -> UIImage {
      
      UIGraphicsBeginImageContextWithOptions(
        snapshot.image.size, true, 0.0)
      
      snapshot.image.draw(at: CGPoint(x: 0, y: 0))
      
      let bezierPath = UIBezierPath()
      guard let firstCoordinate = coordinates.first else {
        fatalError()
      }
      let firstPoint = snapshot.point(for: firstCoordinate)
      bezierPath.move(to: firstPoint)
      
      for coordinates in coordinates.dropFirst() {
        let point = snapshot.point(for: coordinates)
        bezierPath.addLine(to: point)
      }
      
      UIColor.red.setStroke()
      bezierPath.lineWidth = 2
      bezierPath.stroke()
      
      guard let image = UIGraphicsGetImageFromCurrentImageContext() else {
        fatalError()
      }
      UIGraphicsEndImageContext()
      
      return image
  }
}
