//  Created by dasdom on 31.08.19.
//  Copyright © 2019 dasdom. All rights reserved.
//

import UIKit
import CoreLocation

class LocationProvider: NSObject {

  private let locationManager: CLLocationManager
  private let updateHandler: (CLLocation?, Error?) -> Void
  var updating = false

  init(updateHandler: @escaping (CLLocation?, Error?) -> Void) {
    
    locationManager = CLLocationManager()
    self.updateHandler = updateHandler
    
    super.init()
    
    locationManager.delegate = self
    locationManager.distanceFilter = 5
    locationManager.requestWhenInUseAuthorization()
  }
  
  func start() {
    locationManager.startUpdatingLocation()
    locationManager.allowsBackgroundLocationUpdates = true
    updating = true
  }
  
  func stop() {
    locationManager.stopUpdatingLocation()
    locationManager.allowsBackgroundLocationUpdates = false
    updating = false
  }
}

extension LocationProvider : CLLocationManagerDelegate {
  
  func locationManager(
    _ manager: CLLocationManager,
    didChangeAuthorization status: CLAuthorizationStatus) {
    
    switch status {
    case .authorizedWhenInUse:
      print("success")
    default:
      // you will implement the other cases in the
      // exercises
      break
    }
  }
  
  func locationManager(_ manager: CLLocationManager,
                       didUpdateLocations locations: [CLLocation]) {
    
    guard let location = locations.first else { return }
    updateHandler(location, nil)
  }
}
