//  Created by dasdom on 17.11.19.
//  Copyright © 2019 dasdom. All rights reserved.
//

import UIKit
import CoreLocation
import LogStore

class LocationProvider: NSObject, CLLocationManagerDelegate {

  private let locationManager: CLLocationManager
  
  private let updateHandler: (CLLocation) -> Void
  private(set) var updating = false
  
  /*
   
  override init() {
  */
  init(updateHandler: @escaping (CLLocation) -> Void) {
    
    locationManager = CLLocationManager()
    self.updateHandler = updateHandler

    super.init()

    locationManager.delegate = self
    locationManager.distanceFilter = 1
    locationManager.requestWhenInUseAuthorization()
  }
  
  // MARK: - CLLocationManagerDelegate
  
  func locationManager(_ manager: CLLocationManager,
                       didChangeAuthorization status:
                       CLAuthorizationStatus) {
    
    switch status {
    case .authorizedWhenInUse:
      printLog("success")
    case .denied:
      printLog("denied")
      // you will implement this case
      // in the exercises
    default:
      break
    }
  }
  
  func locationManager(_ manager: CLLocationManager,
                       didUpdateLocations locations: [CLLocation]) {
    
    /*
    printLog("locations: \(locations)")
    */
    guard let location = locations.last else {
      return
    }
    updateHandler(location)
  }
  
  // MARK: -
  
  func start() {
    locationManager.startUpdatingLocation()
    updating = true
    locationManager.allowsBackgroundLocationUpdates = true
  }
  
  func stop() {
    locationManager.stopUpdatingLocation()
    updating = false
    locationManager.allowsBackgroundLocationUpdates = false
  }
}
